#!/bin/bash

#############################################################
# 0. Setup
#############################################################

# Choosing the grid
# standards : 96x95x39, 144x142x79
# Horizontal 
horiz_grid=48x36
# Vertical
LM=39
grid_resolution=${horiz_grid}x$LM

an=clim
parallel=1
mpi=2
omp=2
machine=local
rad=rrtm  # Options de rayonnement oldrad|rrtm|ecrad
veg=CMIP6  # Options vegetation NONE|CMIP6
version="20221201.trunk -r 4472"
version=20221201.trunk




if [ ! -f DEF/L$LM.def ] 
    then echo Vertical grid with $LM level not available ; exit
    else sed -i'' -e 's/INCLUDEDEF=L[0-9].*.def/INCLUDEDEF=L'$LM'.def/' DEF/run.def ; fi

hostname=`hostname`
if [ ${hostname:0:5} = ada33 ] ; then
   machine=X64_ADA
fi

# A function to fetch files either locally or on the internet
function myget { #1st and only argument should be file name
  # Path on local computer where to look for the datafile
  if [ -f /u/lmdz/WWW/LMDZ/pub/3DInputData/$1 ] ; then
    \cp -f -p /u/lmdz/WWW/LMDZ/pub/3DInputData/$1 .
  elif [ -f ~/LMDZ/pub/3DInputData/$1 ] ; then
    ln -s ~/LMDZ/pub/3DInputData/$1 .
  else
    wget https://lmdz.lmd.jussieu.fr/pub/3DInputData/$1
    echo wget fini
    dir=~/LMDZ/pub/3DInputData/`dirname $1` ; mkdir -p $dir ; cp -r `basename $1` $dir/
    echo cp finie
  fi
}

#------------------------------------------------------------
# Compilation options for parallel/sequential
#------------------------------------------------------------
modipsl=`pwd | sed -n -e 's/modipsl.*.$/modipsl/p'`

if [ $parallel = 1 ] ; then
   opt_compile="-mem -parallel mpi_omp -arch $machine"
   rungcm="./run_$machine.sh $mpi $omp gcm.e"
   runini="./run_$machine.sh 1 1 ce0l.e"
else
   opt_compile="-arch $machine"
   rungcm="./gcm.e"
   runini="./ce0l.e"
fi

#------------------------------------------------------------
# For Ecrad : donwloading input files and changing iflag_rrtm to 2
#------------------------------------------------------------

case $rad in
   ecrad)  wget https://lmdz.lmd.jussieu.fr/pub/3DInputData/ecrad/ecrad_data.tar
           tar xvf ecrad_data.tar
           iflag_rrtm=2
           NSW=6 ;;
   rrtm)   iflag_rrtm=1
           NSW=6 ;;
   oldrad) iflag_rrtm=0
           NSW=2 ;;
   *)  echo cas non prevu ; exit 1
esac

sed -i'' -e 's/^iflag_rrtm=.*$/iflag_rrtm='$iflag_rrtm'/' -e 's/^NSW=.*$/NSW='$NSW'/' DEF/physiq.def


#------------------------------------------------------------
#  Compilation options for Land-surface model
#------------------------------------------------------------

\rm -f tmp.*
case $veg in
   NONE) optveget="" ; VEGET=n ; veget=0 ;;
   CMIP6) optveget="-v orchidee2.0" ; VEGET=y ; veget=1 ;;
   *) echo VEGET $veget non prevu ; exit
esac


#------------------------------------------------------------
# Model location and installation
#------------------------------------------------------------

if [ `pwd | grep modipsl` ] ; then
   MODEL=`pwd | sed -e 's/.modipsl/ /' | awk ' { print $1 } '`
   LMDZ=$MODEL/modipsl/modeles/LMDZ
   if [ ! -d $LMDZ ] ; then LMDZ=$MODEL/modipsl/modeles/LMDZ5 ; fi
else
    LMDZname=LMDZ`echo $version | sed -e 's/-v//g' -e 's/-unstable//' -e 's/-r/r/' -e 's/ //g'`
    LMDZ=`pwd`/$LMDZname/modipsl/modeles/LMDZ
    if [ ! -d $LMDZ ] ; then
       echo Installing LMDZ version $version
       opti=
       if [ $parallel = 1 ] ; then opti="$opti -parallel mpi_omp" ; fi
       wget https://lmdz.lmd.jussieu.fr/pub/install_lmdz.sh
       chmod +x install_lmdz.sh
       ./install_lmdz.sh -bench 0 -v $version $opti -rad $rad -veget $veg -name $LMDZname
    fi
fi


SIMU=`pwd`
if [ ! -f DEF/run.def -o ! -f DEF/physiq.def -o ! -f DEF/gcm.def ] ; then
    echo Il n y a pas de fichier run.def ou physiq.def ou gcm.def dans le
    echo repertoire $SIMU. Vous pouvez en recuperer sur
    echo $LMDZ/DefLists
    exit
fi

#########################################################################
# 1. Model compilation
#########################################################################

echo
echo '#############################################################'
echo '  1. Model Compilation                                       '
echo '#############################################################'

echo 'Checking the consistency of the compilation and installation'
compilo=`grep COMPIL $LMDZ/arch/arch-local.fcm | awk ' { print $2 } '`
compilo=`basename $compilo`
if [ "$machine" = "local" -a "$compilo" = "mpif90" -a $parallel = 0 ] ; then
    echo You try to work in serial mode while the model was install in parallel
    exit # if you are an expert, you can modify this
fi
if [ "$machine" = "local" -a "$compilo" = "gfortran" -a $parallel = 1 ] ; then
    echo You try to work in parallel mode while the model was install in serial
    exit # if you are an expert, you can modify this
fi

local=`pwd`
suffix=${grid_resolution}_phylmd
if [ `svnversion $LMDZ | egrep -o "[0-9]+"` -ge 4190 ] ; then suffix=${suffix}_$rad ; fi
if [ ${parallel} = 0 ] ; then
    suffix=${suffix}_seq
fi
if [ ${parallel} = 1 ] ; then
    suffix=${suffix}_para_mem
fi
if [ $veg != NONE ] ; then
    suffix=${suffix}_orch
fi
cat <<eod>| compile.sh
prog=\$1
cd $LMDZ
./makelmdz_fcm -j 8 -d $grid_resolution $optveget $opt_compile -rad $rad \$1

cp bin/\${prog}_${suffix}.e $local/\${prog}.e
eod
chmod +x compile.sh

for mod in gcm ce0l ; do
   ./compile.sh $mod
   if [ ! -f $mod.e ] ; then
      echo Echec pour la compilation de lmdz ; exit
   fi
done

#########################################################################
# 2. Creating initial state and boundary conditions
#########################################################################

echo
echo '###############################################################'
echo '  2. Creating initial state and boundary conditions'
echo '###############################################################'
echo '  in files start.nc, startphy.nc, limit.nc             '
echo 
echo '...............................................................'
echo '  2.1  Geting input files from the web                         '
echo '...............................................................'
mkdir -p $SIMU/INITIAL
cd $SIMU/INITIAL
ln -s ../*.e ../*.sh .
cp ../DEF/*def .

if [ "$an" = "clim" ] ; then
   suf=1x1_clim
else
   suf=360x180_$an
fi

echo Input files downloaded from : $get
for file in Albedo.nc Relief.nc Rugos.nc landiceref.nc amipbc_sic_$suf.nc amipbc_sst_$suf.nc ; do
   if [ -f $file ] ; then
# If input files are already there, do not download
      echo File $file is there already
   else
      myget Limit/$file
   fi
done
if [ -f ECDYN.nc ] ; then echo ECDYN.nc already there ; else myget Init/ECDYN.nc ; fi 

if [ -f amipbc_sst_$suf.nc ] ;  then
    ln -sf ECDYN.nc ECPHY.nc
    ln -sf amipbc_sic_$suf.nc amipbc_sic_1x1.nc
    ln -sf amipbc_sst_$suf.nc amipbc_sst_1x1.nc
else
    echo Les fichiers n ont pas ete telecharges correctement ; exit
fi

echo
echo '...............................................................'
echo ' 2.2 Running ce0l.e  (output listing in ce0l.out)              '
echo '...............................................................'
echo "$runini > ce0l.out 2>&1"
$runini > ce0l.out 2>&1

echo
echo '.....................................................'
echo ' 2.3 Creating a figure for the grid named grid.pdf'
echo '.....................................................'
cat <<eod>| tmp.jnl
use grilles_gcm.nc
let rel=if (abs(grille_s-1) gt 0.5) then phis
shade/pal=land_sea_values rel ; go land
let deep=if (rel lt 0.1) then rel
shade/pal=blue/o deep
quit
eod
ferret -batch tmp.ps -nojnl -script  tmp.jnl > out.ferret 2>&1
ps2epsi tmp.ps ; epstopdf tmp.ps ; \mv tmp.pdf grid.pdf >>  out.ferret 2>&1
echo 'If you didn not get grid.pdf, but you have tmp.ps in INITIAL'
echo 'you can try "ps2pdf tmp.ps grid.pdf" or "epspdf tmp.ps grid.pdf",'
echo 'or visualize it directly with "evince tmp.ps" ' 

# Out from INITIAL, back in TUTORIAL :
cd ../

####################################################################
# Preparing a 1-day simulation to initialize orchidee if veget=y
####################################################################
if [ $veg != NONE ] ; then

   echo
   echo '...................................................................'
   echo ' 2.4 : veg!=NONE --> You want to run a simulation with ORCHIDEE -->  '
   echo ' Preparing a 1-day preliminary simulation to create ORCHIDEE initial file'
   echo '...................................................................'
   echo '   Geting input files from the web for Orchidee                '


   mkdir SIMU0 ; cd SIMU0
   for file in cartepente2d_15min.nc routing.nc routing_simple.nc lai2D.nc \
               soils_param.nc alb_bg_modisopt_2D_ESA_v2.nc reftemp.nc \
               PFTmap_15PFT.v1_2000.nc woodharvest_2000.nc ; do
      myget Orchidee/$file
   done
   ln -sf PFTmap_15PFT.v1_2000.nc PFTmap.nc
   ln -s woodharvest_2000.nc woodharvest.nc
   ln -s alb_bg_modisopt_2D_ESA_v2.nc alb_bg.nc

   # Modifying .def files for a first 1-day simulation
   cp -f ../DEF/*def .
   sed -e 's/VEGET=.*.$/VEGET='$VEGET'/' ../DEF/config.def >| config.def
   sed -e 's/nday=.*.$/nday=1/' ../DEF/run.def >| run.def
   sed -e 's/^SECHIBA_restart_in=.*./SECHIBA_restart_in=NONE/' \
       -e 's/^STOMATE_RESTART_FILEIN=.*./STOMATE_RESTART_FILEIN=NONE/' \
                               ../DEF/orchidee.def >| orchidee.def
   if [ $rad = ecrad ] ; then ln -s ../ecrad_data ./ ; cp -f ../DEF/namelist_ecrad . ; fi
   ln -s ../INITIAL/start.nc ../INITIAL/startphy.nc ../INITIAL/limit.nc .
   ln -s ../gcm.e ../*sh .

   # Back to TUTORIAL
   cd ..
fi

####################################################################
# 3. Preparing a Directory for a first simulation :
####################################################################

echo '###############################################################'
echo '  3. Preparing a Directory for a multi-month simulation : PROD0'
echo '###############################################################'

mkdir PROD0
for st in start startphy ; do
	cp INITIAL/$st.nc PROD0/$st.200001.nc ; done
cp INITIAL/limit.nc PROD0/
mkdir PROD0/DEF
for def in config.def gcm.def L$LM.def physiq.def run.def traceur.def namelist_ecrad ; do
	cp DEF/$def PROD0/DEF/$def ; done
if [ $veg != NONE ] ; then
    sed -e 's/^SECHIBA_restart_in=.*./SECHIBA_restart_in=sechiba_rest_in.nc/' \
       -e 's/^STOMATE_RESTART_FILEIN=.*./STOMATE_RESTART_FILEIN=stomate_rest_in.nc/' \
       DEF/orchidee.def > PROD0/DEF/orchidee.def
    for f in sechiba stomate ; do ln -s ../SIMU0/${f}_rest_out.nc PROD0/start_${f}.200001.nc ; done
    sed -e 's/VEGET=.*$/VEGET=y/' -i'' PROD0/DEF/config.def
fi
if [ $rad = ecrad ] ; then cd PROD0 ; ln -s ../ecrad_data ./ ; cd - ; fi

sed -i'' -e 's/nday=.*.$/nday=30/' PROD0/DEF/run.def
echo 200001 afaire >| PROD0/etat
cp run_local.sh gcm.e enchaine.sh reb.sh PROD0
sed -i'' -e 's/veget=.*$/veget='$veget'/' -e 's/parallel=.*$/parallel='$parallel'/' PROD0/enchaine.sh


echo '###############################################################'
echo '  4. Preparing a Directory for a first simulation :            '
echo '###############################################################'


# in TUTORIAL :
mkdir SIMU1
cd SIMU1

# links to executables, *def files, boundary conditions
#------------------------------------------------------
ln -s ../gcm.e ../*.sh .
cp ../DEF/*def .
sed -e 's/VEGET=.*.$/VEGET='$VEGET'/' ../DEF/config.def >| config.def
ln -s ../INITIAL/limit.nc .
if [ $rad = ecrad ] ; then ln -s ../ecrad_data ./ ; cp -f ../DEF/namelist_ecrad . ; fi

# links to initial conditions; 
# NB : for veget!=0, the start file sechiba_rest_in.nc has yet to be produced
##------------------------------------------------------
if [ $veg != NONE ] ; then
   ln -s ../SIMU0/restart.nc start.nc
   ln -s ../SIMU0/restartphy.nc startphy.nc
   for f in sechiba stomate ; do ln -s ../SIMU0/${f}_rest_out.nc ${f}_rest_in.nc ; done
   sed -e 's/^SECHIBA_restart_in=.*./SECHIBA_restart_in=sechiba_rest_in.nc/' \
       -e 's/^STOMATE_RESTART_FILEIN=.*./STOMATE_RESTART_FILEIN=stomate_rest_in.nc/' \
       -i'' orchidee.def
else
   ln -s ../INITIAL/start.nc ../INITIAL/startphy.nc .
fi

# Back to TUTORIAL level
cd ..

####################################################################
# 4. Issuing instructions  :
####################################################################

echo '!------------------------------------!'
echo '!  INSTRUCTIONS TO RUN A SIMULATION  !'
echo '!------------------------------------!'

if [ $veg != NONE ] ; then
   echo ' To run the model with Orchidee, you need to run a preliminary'
   echo ' simulation in directory SIMU0, to create the orchidee initial file'
   echo ' cd SIMU0'
   echo   $rungcm' > listing0                           '
   echo 'Then you can run your simulation in directory SIMU1  '
   echo 'cd ../SIMU1 '
else
   echo ' To run the model without Orchidee :'
   echo 'cd SIMU1' 
fi 
echo   $rungcm' > listing1'
echo 'enjoy it'

